--*****************************************************
-- ORTFixer!&Helper-v0_51.ms
-- "Out-Of-Range Type Fixer!&Helper"
-- v0.51
-- (MAXr3.1, possibly earlier)
-- 12.19.01
-- by swami*, a.k.a. "codeWarrior()", swami@cfl.rr.com
--
--    ***************************************************
--    ***      FOR NON-COMMERCIAL USE ONLY!           ***
--    *** Contact the author for licensing agreement. ***
--    ***************************************************
--
--*****************************************************
-- INSTALLATION:
--    - Put script in MAX ..\Scripts.
--*****************************************************
-- USAGE:
--    - Load a gear train MAX scene (generated using 'Gear-O-Matic!').
--    - Select desired Out-Of-Range Type.
--    - Set 'Match ORT...' checkbox as desired.
--      (NOTE: alt-'Match ORT...' button will set ORT radiobuttons to match drive pinion RotControl ORT)
--    - Set 'Recouple Time Range' checkbox as desired.
--    - Use the 'Pick 'Drive Pinion RotControl'' pickbutton to pick the master driver.
--    - Hit 'Fix Out-Of-Range Types!'
--      (NOTE: This will also fix the scene so it will network render properly. This does slow viewport
--      playback though, so you may want to use 'Remove Net Render Fix', then redo 'Fix Out-Of-Range Types!'
--      just before saving out the scene.)
--    - Done.
--*****************************************************
-- COMMENTS:
--    * This script is to be used with scenes created by 'Gear-O-Matic!'
--    * This script currently only works properly with one gear train in the scene
--      (see 'Future Plans').
--
--    - If the driving pinion RotControl is to do something other than simply spin in one direction at a constant
--      rate (the default in 'Gear-O-Matic'), the Out-Of-Range Types may need to be adjusted to something other than
--      RelativeRepeat. Furthermore, and this is the tricky part, the ORT's must all have their time range bars set
--      to the same range as the driving pinion RotControl! The script does this automatically!
--    NOTE:
--    1) The fix is applied to ALL 'RotControl' objects in the scene, excluding the drive pinion RotControl. This means
--       that only one gear train can exist in the scene, otherwise gears from one train will be adjusted to the wrong
--       drive pinion RotControl (will be fixed in future version).
--    2) The 'before' and 'after' ORTs are assumed to be the same.
--    3) Thanx to Bobo for directing me to the get/set time range functions.
--*****************************************************
-- HISTORY:
--    - v0.51 (12.19.01):
--        - Updated to max R4.20.
--    - v0.5 (07.04.2k):
--        - Major rewrite to handle network rendering issue. Included 'Remove Net Render Fix' option to allow smooth playback
--          in the viewport.
--    - v0.41 (06.24.2k):
--        - Added error checking/handling when attemping to fix ORT without selecting a drive pinion.
--    - v0.4 (06.19.2k):
--        - Changed name from 'ORTFixer!' to 'ORTFixer!&Helper' to reflect expanded functionality.
--        - Added get/set time range for 'RotControl' based on the drive pinion's RotControl time range.
--        - Added 'Constant' to the 'Out-Of-Range Types' radiobuttons list.
--        - Added feature to recouple time ranges.
--    - v0.3 (06.19.2k):
--        - Added option to set Out-Of-Range Type.
--    - v0.2 (06.18.2k):
--        - Added 'ORTF' prefix to 'Usage/Purpose/Comments/Quirks/FuturePlans...' strings so they wouldn't step on
--          other floaters! Tried without prefix, but defining as local, but got error:
--          'Compile error: no local declarations at top level'
--    - v0.1 (06.15.2k):
--        - 1st Beta.
--*****************************************************
--
--             <<< S C R I P T   S T A R T S   H E R E >>> 
--
--
if ORTFH_float != undefined do (closeRolloutFloater ORTFH_float) --If ORTFH_float already open, then close.
--
-- << D e f i n e   V a r i a b l e s   &   A r r a y s >>
ORTFH_floatWidth=310; ORTFH_floatHeight=404

global ORTFH_roll

-- Debug stuff.
DEBUG=format
-- Flag							Associated Action(s)
DB_updateDrivePinionORT=false	-- blahblahblah.
DB_netRendFix=false				-- Write from 'netRendFix' function to Listener.
DB_ORTFixer_but=false			-- Write info during 'ORTFixer' button pressed to Listener.
clearListenerORTFHFlag=true		-- Clear Listener.

-- Strings for 'Purpose/CUsage/omments/Quirks/FuturePlans...' Rollout...
ORTFpurposeString="
--    'Fixer!' (r3.1 only):
--    - To fix a nasty habit of MAX: on loading a scene,
--      objects with script controllers have their Out-Of-Range Types reset to 'Constant'!
--      (this little script sets the appropriate objects' ORT's)
--
--    'Helper':
--    - To set and recouple time range bars, and to set Out-Of-Range Types.
"
ORTFusageString="
--    - Load a gear train MAX scene (generated using 'Gear-O-Matic!').
--    - Select desired Out-Of-Range Type.
--    - Set 'Match ORT...' checkbox as desired.
--      (NOTE: alt-'Match ORT...' button will set ORT radiobuttons to match drive pinion RotControl ORT)
--    - Set 'Recouple Time Range' checkbox as desired.
--    - Use the 'Pick 'Drive Pinion RotControl'' pickbutton to pick the master driver.
--    - Hit 'Fix Out-Of-Range Types!'
--      (NOTE: This will also fix the scene so it will network render properly. This does slow viewport
--      playback though, so you may want to use 'Remove Net Render Fix', then redo 'Fix Out-Of-Range Types!'
--      just before saving out the scene.)
--    - Done.
"
ORTFcommentsString="
--    * This script is to be used with scenes created by 'Gear-O-Matic!'
--    * This script currently only works properly with one gear train in the scene
--      (see 'Future Plans').
--
--    - If the driving pinion RotControl is to do something other than simply spin in one direction at a constant
--      rate (the default in 'Gear-O-Matic'), the Out-Of-Range Types may need to be adjusted to something other than
--      RelativeRepeat. Furthermore, and this is the tricky part, the ORT's must all have their time range bars set
--      to the same range as the driving pinion RotControl! The script does this automatically!
--    NOTE:
--    1) The fix is applied to ALL 'RotControl' objects in the scene, excluding the drive pinion RotControl. This means
--       that only one gear train can exist in the scene, otherwise gears from one train will be adjusted to the wrong
--       drive pinion RotControl (will be fixed in future version).
--    2) The 'before' and 'after' ORTs are assumed to be the same.
--    3) Thanx to Bobo for directing me to the get/set time range functions.
"
ORTFbugsQuirksString="
--    - Q- Only one gear train can exist in the scene.
--    - Please, report Bugs/Quirks to swami@cfl.rr.com, thanx.
"
ORTFfutureString="
--    - Alter script to allow fixing of multiple gear trains.
--    - (DONE!) Script setting of active range bars for 'RotControl' objects.
"

-- Help strings.
netRendFixFlag_nam="Net Render Fix"
netRendFixFlag_des="
Temporary, limited fix to allow network rendering of GOM! generated files.
This 'fix' is required due to a bug in MAX: on loading a scene, objects with
script controllers have their Out-Of-Range Types reset to 'Constant'!\n
It is assumed that the master drive pinion's RotControl uses Relative Repeat
as the Out-Of-Range Type. The script controller will reset the ORT at every
frame. This will slow rendering slightly, but at least it will net render.
"
--------------------------
--<< F U N C T I O N S >>
--------------------------
-- FUNCTION (netRendFix)
--
--    - This function provides a fix to allow network rendering of GOM! generated files.
--    - This 'fix' is required due to a bug in MAX: on loading a scene, objects with script
--      controllers have their Out-Of-Range Types reset to 'Constant'!
--    - The script controller will reset the ORT at every frame. This will slow rendering slightly,
--      but at least it will net render.
--
fn netRendFix obj ORT =
(
	if DB_netRendFix then DEBUG"DB\ Entered 'netRendFix' fn...\n"
	try
	(
		theScript=obj.rotation.local_z_rotation.controller.script
		
		-- 1st attempt at determining if ORTs already set.
		-- 'readLine' generated a "read past end" error, because apparently <CR><LF> is not
		-- recognized in script text! (go figure!?).
--		theScriptAsSS=theScript as stringstream
--		line01=readLine theScriptAsSS
--		if findString line01 "setBeforeORT" != undefined then
--		(
--			readLine theScriptAsSS				-- line02.
--			theScript=readLine theScriptAsSS	-- line03.
--		)

		-- 2nd attempt at determining if ORTs already set.
		if findString theScript "setBeforeORT" != undefined then
		(
			start=findString theScript "(pi"
			end=theScript.count
			theScript=subString theScript start end
		)
		
		theRotControlController="$'"+obj.name+"'.rotation.local_z_rotation.controller"
		theFixString="setBeforeORT "+theRotControlController+" #"+ORT+"\n"+\
		"setAfterORT "+theRotControlController+" #"+ORT+"\n"
		
		theScript=theFixString+theScript
		obj.rotation.local_z_rotation.controller.script=theScript
	)
	catch()
)
--------------------------
-- FUNCTION (removeNetRendFix)
--
-- This function removes the ORT script controller code portion. This speeds playback which is desired
-- during development of a GOM! scene.

fn removeNetRendFix obj =
(
	try
	(
		theScript=obj.rotation.local_z_rotation.controller.script

		-- Taken form '2nd attempt at determining if ORTs already set'.
		if findString theScript "setBeforeORT" != undefined then
		(
			start=findString theScript "(pi"
			end=theScript.count
			theScript=subString theScript start end
			obj.rotation.local_z_rotation.controller.script=theScript
		)
	)
	catch()
)
--------------------------
-- FUNCTION (drivePinion_flt)
--
-- This function filters out objects with "Wheel" AND "RotControl" in the name
-- (used by pickButton destination_pbt).

fn drivePinion_flt obj =
(
	if (findString obj.name "Pinion" != undefined)\
	AND (findString obj.name "RotControl" != undefined)\
	AND obj.rotation.local_z_rotation.keys != undefined
	then drivePinion=true else drivePinion=false
)
--------------------------
-- FUNCTION (updateDrivePinionORT)
--
-- This function updates 'drivePinionORT' which is set to the drive pinion's 'RotControl' Out-Of-Range Type.
-- It also updates the ORTs radio buttons, accordingly.

fn updateDrivePinionORT drivePinion ORTsArray =
(
	if DB_updateDrivePinionORT then DEBUG"DB\ Entered 'updateDrivePinionORT'...\n"
	try
	(
		global drivePinionController=drivePinion.rotation.controller.local_z_rotation.controller
		global drivePinionORT=getBeforeORT drivePinionController
		if DB_updateDrivePinionORT then DEBUG"DB\ drivePinionORT=%\n" drivePinionORT
		if ORTFH_roll.matchORT_cbx.checked then
		(
			for i = 1 to ORTsArray.count do
			(
				if DB_updateDrivePinionORT then DEBUG"DB\ i:%, (ORTsArray[i] as name)=%\n" i (ORTsArray[i] as name)
				if (ORTsArray[i] as name)==drivePinionORT then
				(
					ORTFH_roll.ORTs_rbt.state=i
					exit
				)
			) 
		)
	)
	catch()
)
--------------------------
-- << C r e a t e   F l o a t e r >>
ORTFH_float = NewRolloutFloater "Out-Of-Range Type Fixer!&Helper v0.51" ORTFH_floatWidth ORTFH_floatHeight
--------------------------
-- << C r e a t e   R o l l o u t s  &  D e f i n e   E v e n t   H a n d l e r s >>
Rollout about_roll "About";
(
	label About01 "Out-Of-Range Type Fixer!&&Helper"		
	label About02 "v0.51"
	label About03 "(MAX r3.1 thru r4.20)"
	label About04 "12.19.01"
	label About05 "by swami*"
	label About06  "a.k.a. \"codeWarrior()\""
	label About07  "swami@cfl.rr.com"
	bitmap Swami fileName:"SwamiDelic!.jpg"
	label About08  ">>> FOR NON-COMMERCIAL USE ONLY <<<"
	label About09  "(Contact swami* for licensing agreement)"
	label About10  "(Contact swami* for custom scripting!)"
)-- Close About Rollout
--------------------------
Rollout usage_roll "Purpose/Usage/Comments/Quirks/FuturePlans..."
(
--<< U I   E l e m e n t s >>
	button purpose_but "Purpose"
	button usage_but "Usage"
	button comments_but "Comments"
	button bugsQuirks_but "Known Bugs/Quirks"
	button future_but "Future Plans..."

--<< E v e n t   H a n d l e r s  >>

	--<< S h o w   U s a g e >>
	on usage_but pressed do
	(
		messagebox ORTFusageString title:"Usage:"
	)

	--<< S h o w   P u r p o s e >>
	on purpose_but pressed do
	(
		messagebox ORTFpurposeString title:"Purpose:"
	)

	--<< S h o w   C o m m e n t s >>
	on comments_but pressed do
	(
		messagebox ORTFcommentsString title:"Comments:"
	)

	--<< K n o w n   B u g s / Q u i r k s >>
	on bugsQuirks_but pressed do
	(
		messagebox ORTFbugsQuirksString title:"Known Bugs/Quirks:"
	)

	--<< F u t u r e   P l a n s >>
	on future_but pressed do
	(
		messagebox ORTFfutureString title:"Future Plans:"
	)

)-- Close usage_roll
--------------------------
drivePinionString="Object Selected: "
ORTsArray=#("RelativeRepeat","Constant","Loop","Cycle","PingPong")
butSize=14
matchORTMsg="
Just what it sez!
"
recoupleMsg="
The drive pinion RotControl controller's active time range will be
'recoupled' to the controller's first and last existing keys. This
sets the range to start at the first key and end at the last.
"
Rollout ORTFH_roll "Main"
(
--<< U I   E l e m e n t s >>

	group "Setup:"
	(
		radiobuttons ORTs_rbt "Out-Of-Range Types:" labels:ORTsArray columns:2
		checkbox matchORT_cbx "Match ORT to drive pinion RotControl" offset:[20,0] checked:true
		button matchORT_but align:#left width:butSize height:butSize offset:[0,-20] tooltip:":Description; alt:Match ORT"
		checkbox recoupleTimeRange_cbx "Recouple Time Range" offset:[20,0] checked:true
		button recoupleTimeRange_but align:#left width:butSize height:butSize offset:[0,-20]
-- Put these back in, later, so option to remove 'net render' mod can be coded...
--		checkbox netRendFixFlag_cbx "Net Render Fix" checked:true align:#left offset:[20,0]
--		button netRendFixFlag_but width:butSize height:butSize align:#left offset:[0,-20] --tooltip:tooltipSpn
		pickbutton drivePinion_pbt "Pick 'Drive Pinion RotControl'" offset:[0,5] filter:drivePinion_flt
		label drivePinion_lab "replaced on open..."
	)
	group "Do It!:"
	(
		button ORTFixer_but ">>> Fix Out-Of-Range Types! <<<" width:175 height:50 offset:[0,5]
		button removeNetRendFix_but "Remove Net Render Fix" offset:[0,5] tooltip:"Speeds viewport playback."
	)

--<< E v e n t   H a n d l e r s  >>

	on ORTFH_roll open do
	(
		drivePinion_lab.text=drivePinionString+"None"
		ORTs_rbt.enabled=NOT matchORT_cbx.checked
		global drivePinion=undefined
		global drivePinionORT=undefined
	)
	
	on matchORT_but pressed do
	(
		case of
		(
			--<< S h o w   D e s c r i p t i o n >> 
			(NOT keyboard.shiftPressed AND NOT keyboard.controlPressed AND NOT keyboard.altPressed):
				(
					messagebox matchORTMsg title:"Match ORT to drive pinion RotControl" tooltip:"Description"
				)
			--<< N o t   U s e d >> (ctrl)
			(keyboard.controlPressed):
				(

				)
			--<< U p d a t e   D r i v e   P i n i o n   O R T   R a d i o b u t t o n s >> (alt)
			(keyboard.altPressed):
				(
					if matchORT_cbx.checked then
					(
						if drivePinion != undefined then
						(
							updateDrivePinionORT drivePinion ORTsArray
							messagebox ("ORT radiobuttons udated.\n\nORT: "+(drivePinionORT as string)) title:"Process report"
						)
						else
						(
						messagebox "Drive Pinion RotControl not yet selected => ORT radiobuttons NOT updated." title:"Process report"
						)
					)
					else
					(
						messagebox "Match ORT is NOT checked => ORT radiobuttons NOT updated." title:"Process report"
					)
				)
		)
	)
	
	on matchORT_cbx changed state do
	(
		ORTs_rbt.enabled=NOT matchORT_cbx.checked 
		try (if matchORT_cbx.checked then updateDrivePinionORT drivePinion ORTsArray);catch()
	)
	
	on drivePinion_pbt picked obj do
	(
		global drivePinion=obj
		drivePinion.isSelected=true
		drivePinion_lab.text=drivePinionString+obj.name
		updateDrivePinionORT drivePinion ORTsArray
	)
	
	on recoupleTimeRange_but pressed do
	(
		messagebox recoupleMsg title:"Description: Recouple Time Range" tooltip:"Description"
	)
		
	on netRendFixFlag_but pressed do
	(
		case of
		(
			--<< D e s c r i p t i o n >> 
			(NOT keyboard.shiftPressed AND NOT keyboard.controlPressed AND NOT keyboard.altPressed):
				(
					theTitle=baseTitle+netRendFixFlag_nam
					messagebox netRendFixFlag_des title:theTitle
				)
			--<< ?????????????????? >> (ctrl)
			(keyboard.controlPressed):
				(

				)
			--<< ?????????????????? >> (alt)
			(keyboard.altPressed):
				(
--					STUB
				)
		)
	)

	on ORTFixer_but pressed do
	(
		setWaitCursor()
		
		if DB_ORTFixer_but then DEBUG"DB\ Entered 'ORTFixer'...\n"
		try
		(
			ORT=ORTsArray[ORTs_rbt.state]
			ORTasName=ORTsArray[ORTs_rbt.state] as name
			drivePinionController=drivePinion.rotation.controller.local_z_rotation.controller
			setBeforeORT drivePinionController ORTasName
			setAfterORT drivePinionController ORTasName
			if recoupleTimeRange_cbx.checked then setTimeRange drivePinionController #linkToKeys
			drivePinionTimeRange=getTimeRange drivePinionController
			
			for obj in objects\
			where
			(
				classOf obj == Dummy\
				AND (findString obj.name "RotControl" != undefined)\
				--AND (obj.rotation.local_z_rotation.controller==float_Script)
			)
			do
			(
				if DB_ORTFixer_but then DEBUG"DB\ In loop: obj=%\n" obj
				netRendFix obj ORT

				objRCController=obj.rotation.local_z_rotation.controller
				setTimeRange objRCController drivePinionTimeRange
			)
			messagebox "DONE" title:"Status Report"
		)
		catch (if DB_ORTFixer_but then DEBUG"DB\ ERROR in loop: obj=%\n" obj)
	)
	
	on removeNetRendFix_but pressed do
	(
		try
		(
			for obj in objects\
			where
			(
				classOf obj == Dummy\
				AND (findString obj.name "RotControl" != undefined)\
				--AND (obj.rotation.local_z_rotation.controller==float_Script)
			)
		 	do
			(
				removeNetRendFix obj
			)
			messagebox "DONE" title:"Status Report"
		)
		catch()
		
	)
	
)-- End ORTFH_roll Rollout
--------------------------
-- << A d d   R o l l o u t s >>
addRollout about_roll ORTFH_float rolledUp:true
addRollout usage_roll ORTFH_float rolledUp:true
addRollout ORTFH_roll ORTFH_float

if clearListenerORTFHFlag then clearListener()
